#!/bin/bash

#debug=1

echo "Testing dialog(1) and awk(1)... This may take a few seconds."

. lib/test-dialog.sh
. lib/test-awk.sh

test_dialog dialog

if test "$?" != "0"
then
	echo "dialog(1) failed. Please install a working version before running $0!"
	exit
fi

test_awk awk

if test "$?" != "0"
then
	echo "awk(1) failed. Please install a working version before running $0!"
	exit
fi

tmpfile="menuconfig.tmp";

if test -f "gpmi.conf"
then
	do_load=yes
fi

awk -v "tmpfile=$tmpfile" -v "do_load=$do_load" -v "debug=$debug" '
BEGIN {
	last_level=1;

	backtitle="GPMI config"
}

#read in config directives
/^#.[ 	]*-*/ {
	t=$1
	sub("^#", "", t)
	level=length($2);
	if (level<last_level) {
		for(n=level+1; n<=last_level; n++) {
			C[n]=0;
		}
	}
	$1="";
	$2="";
	C[level]++
	
	place=""
	for(n=1; n<=level; n++) {
		if (n==1) 
			place=C[n]
		else
			place=place "_" C[n]
	}

	cmt=$0
	sub("^ *", "", cmt)	
	if (debug)
		print "comment[" place "]=\"" cmt "\"| t=" t
	COMMENT[place]=cmt
	TYPE[place]=t
	last_level=level
	next
}

function set_var() {
		split($0, A, "=");
		NAME[place]=A[1]
		VALUE[place]=A[2]
		gsub("\"", "", VALUE[place]);
		if (debug)
			print " " place "-->  |" A[1] "| := |"	A[2] "|"
}

# take the variable
/^[^#].*=/{
	set_var();
}

function back(pos) {
	if (match(pos, "_")==0)
		return "0"
		
	sub("_[1234567890]*$", "", pos);
	return pos;
}

function assamble_(type, comment, address, len, title					,res,n,a) {
#	res="--backtitle  \"" backtitle "\""
	res=""
	if ((type == "M") || (type == "I")) {
		res=res " --menu \"" title "\" 20 70 13 "
		for(n=1; n<len; n++) {
			res=res " \"" address[n] "\" \"" comment[n] "\" " 
		}
	}
	else if (type == "C") {
		last_value=""
		res=res " --checklist \"" title "\" 20 70 13 "
		for(n=1; n<len; n++) {
			a=address[n]
			if (VALUE[a] == 1) {
				v="on"
				last_value=last_value a " "
			}
			else
				v="off"
			VALUE[a]=0;				
			res=res " " a " \"" comment[n] "\" " v " "
		}
	}
	else if (type == "R") {
		last_value = VALUE[address[0]]
		res=res " --radiolist \"" title "\" 20 70 13 "
		for(n=1; n<len; n++) {
			if (VALUE[address[0]] == VALUE[address[n]])
				v="on"
			else
				v="off"
			res=res " " address[n] " \"" comment[n] "\" " v " "
		}
		last_place=address[0]
	}
	last_len=len
	last_type=type
	return res
}


function assamble(    n,l,p,c,t,a) {
	place=""
	for(n=1; n<level; n++) {
		if (n==1) 
			place=POS[n]
		else
			place=place "_" POS[n]
	}
	
	if (level == 1)
		title="Main menu"
	else
		title=COMMENT[place]

	if (debug)
		print "Place=" place
  
	a[0]=place;
	
	for(l=1; ;l++) {
		if (level==1) 
			p=l
		else
			p=place "_" l
		c[l]=COMMENT[p]
		a[l]=p;
		if (debug)
			print "p=" p " | c=" c[l]
		if (l==1) 
			t=TYPE[p]
		if (c[l]=="") {
			if (level == 1) {
				c[l]="-"
				a[l]="-"
				l++;
				c[l]="search a setting"
				a[l]="SEARCH"
				l++;
				c[l]="save to alternate file"
				a[l]="SAVE"
				l++;
				c[l]="load from alternate file"
				a[l]="LOAD"
				l++;
				c[l]="run configure"
				a[l]="RC"
				l++;
				c[l]="quit"
				a[l]="QUIT"
				l++;
			}
			else {
				if ((t=="M") || (t=="I")) {
					c[l]="<-- back"
					a[l]=back(place)
					l++;
				}
			}
			return assamble_(t, c, a, l, title);
		}
	}
}

function process_return_() {
			if ($0==0)
				level=1
			else if ($0 == "QUIT") {
				if (yesno("Do you want to save current settings in gpmi.conf?") == 0) {
					print "Saving settings to gpmi.conf"
					save_("gpmi.conf");
				}
				print "Aborted"
				exit
			}
			else if ($0 == "LOAD") {
				load()
			}
			else if ($0 == "SEARCH") {
				search()
			}
			else if ($0 == "SAVE") {
				save()
			}
			else if ($0 == "RC") {
				save_("gpmi.conf");
				configure("gpmi.conf");
				exit
			}
			else {
				level=split($0, POS, "_")		
				level++;	
			}
}

function process_return() {
		if (last_type == "M") {
			process_return_();
		}
		else if (last_type == "I") {
			old_level=level;
			process_return_();
			if (level>old_level) {
				varname=$0
				gsub("\"", "", varname)
				inputbox(NAME[varname], VALUE[varname]);
				VALUE[varname]=$0
				level--;
			}
		}
		else if (last_type == "C") {
			gsub("[\"]", "", $0)
			for(n=1; n<=NF; n++) {
				VALUE[$n]=1
			}
			level--;
		}
		else if (last_type == "R") {
			gsub("[\"]", "", $0)
			VALUE[last_place]=VALUE[$0]
			level--;
		}
}

function search(			res, A, n) {
		inputbox("Setting name to search for:", search_str);
		search_str=$0
		res=" --menu \"search results\" 20 70 13 "
		for(n in NAME) {
			if ((match(NAME[n], search_str)!=0) || (match(VALUE[n], search_str)!=0) || (match(COMMENT[n], search_str)!=0))
				res = res  n " " NAME[n] " "
		}
		if (!run_dialog(res)) {
			default_item=$0 
			level=split($0, POS, "_")
		}
}

function process_cancel() {
	if (last_type == "C") {
		$0="- " last_value
		process_return()
	}
	else if (last_type == "R") {
		VALUE[last_place]=last_value
		level--;
	}
	else level--;
}


function run_dialog(params			,res) {
	if (debug)
		print "Calling dialog with " params
	if (default_item != "") {
		di = "--default-item " default_item
		default_item = ""
	}
	else di=""
	
	res=system("dialog " di " " params " 2>" tmpfile)
	close(tmpfile)
	getline < tmpfile
	return res
}

function run_dialog_retval(params) {
	if (debug)
		print "Calling dialog with " params
	return system("dialog " params )
}

function inputbox(title, value) {
	gsub("\"", "", title)
	gsub("\"", "", value)
	run_dialog("--backtitle  \"" backtitle "\" --inputbox \"" title "\" 8 70 \"" value "\"")
}

function yesno(title) {
	gsub("\"", "", title)
	gsub("\"", "", value)
	return run_dialog_retval("--backtitle  \"" backtitle "\" --yesno \"" title "\" 8 70")
}

function load_(fn					,l) {
	for(l=0; (getline < fn)!=0; l++) {
		if (match($1, "^#") == 0)
			set_var();
		else
			place=$2
	}
	return l;
}

function save_(fn					,n) {
	close(fn)
	print "# settings for gpmi configure.sh; generated and used by menuconfig.sh" > fn
	for(n in NAME) {
		print "# " n >> fn
		print NAME[n] "=" VALUE[n]  >> fn
	}
	close(fn)
}

function save() {
	inputbox("Alternate file name to save config to", "gpmi_alt.conf")
	save_($0);
}


function load() {
	inputbox("Alternate file name to load config from", "gpmi_alt.conf")
	load_($0);
}

function configure(fn) {
	system("./configure.sh --include=" fn);
}

#actual config
END {

	if (do_load=="yes") {
		print "Loading gpmi.conf..."
		if (load_("gpmi.conf")>0) {
			print "Done."
		}
	}
	
	level=1;
	while (1) {
		if (!run_dialog(assamble()))
			process_return();
		else
			process_cancel();
			
		if (debug)
			print "Entering config level " level
	}
}

' < configure.defaults

rm -f $tmpfile

#echo ${A[2,2]}