/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
 * @file global.h
 * \brief The global include-file.
 *
 * Handles the safe-macros to call module-procs, and makes gpmi-core functions available.
 */

#ifndef GPMI_GLOBAL_H
#define GPMI_GLOBAL_H

#define PREFIX_GPMI_PKG_GEN "GPMI_PKG_GEN_"
#define PREFIX_GPMI_PKG_GEN_LEN 13

extern int gpmi_initialize_paths; /* Do not use unless you know its effects!! */
extern int gpmi_initialized;      /* Don't use it unless you really know how it works */
extern void gpmi_init(void);
extern void gpmi_uninit(void);

typedef struct gpmi_module gpmi_module;
typedef struct gpmi_package gpmi_package;
typedef struct gpmi_globev_param_t gpmi_globev_param_t;

typedef void (*gpmi_global_event_t)(gpmi_module *mod, int event_id, int argc, gpmi_globev_param_t *argv_in);

/**
 * \brief THE way to call RunTick of a module.
 *
 * This macro does some safe-checking before calling the proc, making sure your
 *  application doesn't crash on a bad module.
 */
#define gpmi_call_RunTick(module_handle, frame_counter) \
	if (module_handle->proc_RunTick != NULL) { \
		gpmi_set_current_module(module_handle); \
		module_handle->proc_RunTick(module_handle, frame_counter); \
	}

/**
 * \brief THE way to call RunData of a module.
 *
 * This macro does some safe-checking before calling the proc, making sure your
 *  application doesn't crash on a bad module.
 */
#define gpmi_call_RunData(module_handle, data) \
	if (module_handle->proc_RunData != NULL) { \
		gpmi_set_current_module(module_handle); \
		module_handle->proc_RunData(module_handle, data); \
	}

/**
 * \brief THE way to call Debug of a module.
 *
 * This macro does some safe-checking before calling the proc, making sure your
 *  application doesn't crash on a bad module.
 */
#define gpmi_call_Debug(module_handle, str) \
	if (module_handle->proc_Debug != NULL) { \
		gpmi_set_current_module(module_handle); \
		module_handle->proc_Debug(module_handle, str); \
	}

/* Below, stuff for NAMESPACE, nothing important */

/* XXX -- Please don't ask why there is a double layer in here, but else NAMESPACE doesn't get resolved... */
#define PACKAGE_FUNCTION_REAL_SECOND(name, ns) GPMI_PKG_USR_ ## ns ## _ ## name
#define PACKAGE_FUNCTION_REAL_FIRST(name, ns) PACKAGE_FUNCTION_REAL_SECOND(name, ns)

#define PACKAGE_FUNCTION(name) PACKAGE_FUNCTION_REAL_FIRST(name, NAMESPACE)
#define PKG_FUNC(name) PACKAGE_FUNCTION(name)

#endif /* GPMI_GLOBAL_H */
