/*
    hash test 1.0.0 - test program for the simple hash lib
    Copyright (C) 2004 Tibor `Igor2` Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Author: igor2@maffia.hu
	    http://igor2.mcomp.hu


*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "hash.h"

int size=3, size2=10, size3=32;

struct hash_t *h;

char *n_en[]={"zero",  "one", "two",   "three", "four", "five", "six", "seven", "eight", "nine",   "ten", "eleven",   "twelve",     "dog",   "cat",    "fish", NULL};
char *n_hu[]={"nulla", "egy", "ketto", "harom", "negy", "ot",   "hat", "het",   "nyolc", "kilenc", "tiz", "tizenegy", "tizenketto", "kutya", "macska", "hal",  NULL};

void deb(char *key)
{
	printf("%s: %s\n",key, (char *)hash_find(h,key));
}

int load()
{
	int n;

	for(n=0; n_en[n]!=NULL; n++) {
		hash_store(h,n_en[n],n_hu[n]);
	}

	return 0;
}

int test()
{
	int n,r;
	char *s;
	for(r=0,n=0; n_en[n]!=NULL; n++) {

		s=hash_find(h,n_en[n]);
		if ((s==NULL) || (strcasecmp(s,n_hu[n])!=0)) {
			printf("ERROR: hash_find %s, expected %s, got %s\n",n_en[n],n_hu[n],s);
			r++;
		}
	}

	s=hash_find(h,"lofasz");

	if (s!=NULL) {
		printf("ERROR: hash_find lofasz, expected NULL, got %s\n",s);
		r++;
	}

	if (r == 0)
		printf("OK: test ok.\n");

	return r;
}

int stat()
{
	int recno, empty_nodes,max_rec_per_node,mem;
	float avg_rec_per_node;
	int r;

	r=hash_stat(h, &recno, &empty_nodes, &avg_rec_per_node, &max_rec_per_node,&mem);

	if (r!=0)
		return 1;

	printf("Stat: size=%d recno=%d empty_nodes=%d avg=%f max=%d memory=%d\n",h->tbl_size,recno,empty_nodes,avg_rec_per_node,max_rec_per_node,mem);
	return 0;
}

int create()
{
	h=hash_create(size,NULL);
	if (h == NULL) {
		printf("ERROR: hash_create failed\n");
		exit(1);
	}
	return 0;
}

int rehash(int size)
{
	h=hash_rehash(h,size,NULL);
	if (h==NULL) {
		printf("ERROR: rehash failed\n");
		exit(2);
	}
	return 0;
}

int destroy()
{
	return hash_destroy(h);
}

int main()
{
	int r;

	r  = create();
	r += load();
	r += test();
	r += stat();

	r += rehash(size2);
	r += test();
	r += stat();

	r += rehash(size3);
	r += test();
	r += stat();

	r += destroy();

	return r;
}
