/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef GPMI_LIBS_MODULE_H
#define GPMI_LIBS_MODULE_H

struct gpmi_module {
	gpmi_library *library;                  //! Underlying library

	char *params;                           //! Params that need to be passed to the AI
	char *path;                             //! Module path - useful when the module wants to open the data dir located near the module binary

	gpmi_ProcInit        *proc_Init;        //! Proc to initialize function (Optional)
	gpmi_ProcUnInit      *proc_UnInit;      //! Proc to uninitialize function (Optional)
	gpmi_ProcRunTick     *proc_RunTick;     //! Proc to RunTick function (timer based applications)
	gpmi_ProcRunData     *proc_RunData;     //! Proc to RunData function (filter applications)
	gpmi_ProcDebug       *proc_Debug;       //! Proc to debugcommand functions (Optional)

	gpmi_global_event_t  proc_EventGlobal;  //! If not NULL, any event is passed here; preparsed call type
	gpmi_ProcEvent       **proc_EventArray; //! If proc_EventGlobal is not NULL, this and event_num are used
	int                  event_next_free;   //! used size of proc_EventArray
	int                  event_allocated;   //! allocated size of proc_EventArray

	void                 *mod_data;         //! module data: the module should store a pointer here to a strucutre where it stores its data
};

/* Functions available for public */
gpmi_module *gpmi_mod_load(const char *library_name, const char *params);
void gpmi_mod_unload(gpmi_module *module);

/* Functions to manipulate "current module" (thread safe) */
gpmi_module *gpmi_get_current_module(void);
void gpmi_set_current_module(gpmi_module *module);


/* Low level functions */
int gpmi_load_module(gpmi_module *module, const char *library_name);

#endif /* GPMI_LIBS_MODULE_H */
