/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & glx & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GPMI_OS_DEP_H
#define GPMI_OS_DEP_H

#ifdef WIN32
char *basename(char *path);
char *dirname(char *path);
#endif

#ifdef DLCOMPAT
#	include <dl-compat.h>
#else
#	include <dlfcn.h>
#endif
#include <limits.h>

/* Define a library handle */
#define gpmi_library_handle void *

#ifndef PATH_MAX
#ifdef MAX_PATH
#define PATH_MAX MAX_PATH
#else
#define PATH_MAX 256
#endif
#endif

#ifdef WIN32
#ifdef __CYGWIN__
#define PRINTF_PTR "%p"
#define PRINTF_PTR_CAST
#else
#define PRINTF_PTR "0x%x"
#define PRINTF_PTR_CAST (size_t)
#endif
#define GPMI_PATH_SEP_STR ";"
#define GPMI_PATH_SEP_CHAR ';'
#else
#define PRINTF_PTR "%p"
#define PRINTF_PTR_CAST
#define GPMI_PATH_SEP_STR ":"
#define GPMI_PATH_SEP_CHAR ':'
#endif

#ifdef UNIX
#include <libgen.h>
#endif

#ifndef OS_DEP_MACROS_ONLY

/* These are the actual installation paths. They can be global variables since
we load them only once in gpmi_init. Never modify them. They can't be constants
because on windows we need to consult the registry to get these. */
char *osdep_GPMI_INSTALL_PATH_USER_MOD_LIB;
char *osdep_GPMI_INSTALL_PATH_USER_PKG_LIB;
char *osdep_GPMI_INSTALL_PATH_MOD_LIB;
char *osdep_GPMI_INSTALL_PATH_PKG_LIB;

typedef enum gpmi_env_e {
	gpmi_env_home,
} gpmi_env_t;

/* Low level functions */
void  gpmi_library_add_path(const char *dir, const char *library_name, char *lib_file_name, char *dep_file_name, int path_size);
int   gpmi_library_load_lowlevel(gpmi_library *library, char *file_name);
void  gpmi_library_unload_lowlevel(gpmi_library *library);
void *gpmi_dlsym_lowlevel(gpmi_library *library, const char *name);
void *gpmi_dlopen_global();
char *gpmi_getenv(gpmi_env_t env);

void gpmi_path_init();
void gpmi_path_uninit();

#else
/* Make header-file free to readd */
#	undef GPMI_OS_DEP_H
#endif

#endif /* GPMI_OS_DEP_H */
