/*
    libgpmi - General Package/Module Interface - gpmi_scripts package
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GPMI_SCRIPTS_H
#define GPMI_SCRIPTS_H

#include <gpmi.h>

typedef int GPMI_Script_Info_Init(void **client_data);
typedef void GPMI_Script_Info_Destroy(void **client_data);
typedef char *GPMI_Script_Info_GetExtension(void);
typedef void GPMI_Script_Info_Register(void *client_data, const char *name);
typedef int GPMI_Script_Info_LoadScript(void *client_data, const char *filename);
typedef int GPMI_Script_Info_Rehash(void *client_data);
typedef void GPMI_Script_Info_RunFunction(void *client_data, const char *function, int argc, const char **argv);
typedef void GPMI_Script_Info_SetPkg(void *client_data, const gpmi_package *package);

typedef struct GPMI_Script_Info {
	GPMI_Script_Info_Init *init;
	GPMI_Script_Info_Destroy *destroy;
	GPMI_Script_Info_GetExtension *get_extension;
	GPMI_Script_Info_Register *reg;
	GPMI_Script_Info_LoadScript *load_script;
	GPMI_Script_Info_Rehash *rehash;
	GPMI_Script_Info_RunFunction *run_function;
	GPMI_Script_Info_SetPkg *set_pkg;

	int function_lowercase; //! Set this to one if the module should get all the function in lowercase
} GPMI_Script_Info;

/* Private functions, only use those in a script-module */

int  GPMI_Script_Load(const GPMI_Script_Info *info, gpmi_module *module, const char *params);
void GPMI_Script_Destroy(gpmi_module *module);
void GPMI_Script_RunTick(gpmi_module *module, int frame_counter);
void GPMI_Script_RunData(gpmi_module *module, void *data);
const void *GPMI_Script_FindProc(const char *function);
void *GPMI_Script_GetClientData();

/* Published functions to be used everywhere */

int Bind(int event, char *script);
//! Bind an event to a function
int Unbind(int event, char *function);
//! Unbinds an event from a function
int FakeEvent(int event, char *param);
//! Fake an event-call with params
int Rehash();
//! Reload the script
char *GetScriptDir();
//! Get the dir of the script
int PkgLoad(char *pkg_name, int version);
//! Load a package
void *AddTimer(int TTL, int freq, char *script, char *user_data);
//! Add a timer to call a function 'TTL' times with 'freq' pauses; returns the new timer's pointer
int DelTimer(void *timer_ptr);
//! Deletes a timer; returns 0 on success
char *TimerInfo(void *timer_data, char *field);
//! returns a field of a timer data structure
void gpmi_puts(const char *s);
//! put a string on stdout, add newline
void gpmi_puts_nonl(const char *s);
//! put a string on stdout, don't add newline
#endif /* GPMI_SCRIPTS_H */
