/*
    libgpmi - General Package/Module Interface - listgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "listgen.tab.h"
#include "listgen1.h"


char *direct = "DIRECT";

int enum_nowrap = 0;
int in_struct = 0;
int verbose = 0;
func_t *functions = NULL;
event_t *events = NULL;
keyword_t *keywords = NULL;

param_t **curr_params = NULL;
faketype_t *faketypes = NULL;
param_stack_t *param_stack = NULL;
enum_t *enums = NULL;


extern char *yylgtext;
extern void yylgparse();

void yylgerror(char *s)
{
	fprintf(stderr, "Parser error: %s at %d:%d near \"%s\"\n", s, yylglloc.first_line+1, yylglloc.first_column+1, yylgtext);
	/* We need to add one to column and line since editors like to start counting from 1 instead of 0 */
	exit(1);
}

int yylgwrap(void)
{
	return 1;
}

char *type_norm(char *type)
{
	char *s, *ret;
	char pref[16];
	int l;

	s = type;
	l = 0;
	while((*s == '!') || (*s == '^') || (*s == '@')) {
		pref[l] = *s;
		s++;
		l++;
	}
	pref[l] = 0;

	     if (strcmp(s, "char*") == 0)		s = "string";
	else if (strcmp(s, "void*") == 0)		s = "ptr";
	else if (strcmp(s, "long") == 0)  	s = "int";

	ret = malloc(strlen(s) + l + 2);
	sprintf(ret, "%s%s", pref, s);
	return ret;
}

void faketype_add(char *name, char *type)
{
	faketype_t *ft;

	ft = malloc(sizeof(faketype_t));
	ft->name  = strdup(name);
	ft->type  = strdup(type);
	ft->next  = faketypes;
	faketypes = ft;
}


void func_add(char *type, char *name)
{
	func_t *f;

	if (in_struct != 0)
		return;

	f = malloc(sizeof(func_t));

	f->type   = type_norm(type);
	f->name   = strdup(name);
	f->params = NULL;
	f->next   = functions;

	functions   = f;
	curr_params = &(f->params);
}

void func_add_direct(char *name)
{
	func_t *f;

	if (in_struct != 0)
		return;

	f = malloc(sizeof(func_t));

	f->type   = direct;
	f->name   = strdup(name);
	f->params = NULL;
	f->next   = functions;

	functions   = f;
	curr_params = NULL;
}

void event_add(char *name)
{
	event_t *e;

	e = malloc(sizeof(event_t));

	e->name   = strdup(name);
	e->params = NULL;
	e->next   = events;

	events      = e;
	curr_params = &(e->params);
}

void keyword_add(char *name, int type, char *type_name)
{
	keyword_t *k;

	k = malloc(sizeof(keyword_t));

	k->name   = strdup(name);
	k->params = NULL;
	k->type   = type;
	k->next   = keywords;

	keywords    = k;
	curr_params = &(k->params);

	if (type_name != NULL)
		faketype_add(type_name, name);

}

void param_add(char *type, char *name)
{
	param_t *p;

	if  (curr_params == NULL)
		return;

	p = malloc(sizeof(param_t));

	p->type = type_norm(type);
	p->name = strdup(name);
	p->next = *curr_params;

	*curr_params = p;
}

keyword_t *find_keyword(char *name)
{
	keyword_t *k;

	for(k = keywords; k != NULL; k = k->next) {
		if (strcmp(k->name, name) == 0)
			return k;
	}

	return NULL;
}

faketype_t *find_faketype(char *name)
{
	faketype_t *ft;

	for(ft = faketypes; ft != NULL; ft = ft->next) {
		if (strcmp(ft->name, name) == 0)
			return ft;
	}

	return NULL;
}

int type_or_kw_depth = 0;
inline char *type_or_kw(char *type)
{
	char *typ;
	faketype_t *fake;
	char *ret, *tmp;
	int prefix_len;

	type_or_kw_depth++;

	if (type_or_kw_depth > 32) {
		fprintf(stderr, "FATAL: Too deep recursion in type_or_kw(). Probably you have cyclic FAKETYPEs.\n");
		exit(1);
	}

	typ = type;

	while ((*typ == '!') || (*typ == '^') || (*typ == '@'))
		typ++;

		if ((strcmp(typ, "int") == 0) || (strcmp(typ, "string") == 0) ||
				(strcmp(typ, "double") == 0) || (strcmp(typ, "void") == 0) ||
				(strcmp(typ, "uint") == 0) || (strcmp(typ, "pre") == 0) ||
				(strcmp(typ, "uchar") == 0) || (strcmp(typ, "char") == 0)
			 ) {
			type_or_kw_depth--;
			return type;
		}

		if (find_keyword(typ) != NULL) {
			type_or_kw_depth--;
			return type;
		}

		fake = find_faketype(typ);
		if (fake != NULL) {
			if (typ != type) {
				prefix_len = typ - type;
			 	ret = malloc(128);
				strncpy(ret, type, prefix_len);
				tmp = type_or_kw(fake->type);
				strncpy(ret + prefix_len, tmp, 128 - prefix_len);
				type_or_kw_depth--;
				/* XXX: this leaks */
				return ret;
			}
			ret = type_or_kw(fake->type);
			type_or_kw_depth--;
			return ret;
		}

		/* Enums are ints. */
		if (find_enum(typ) != NULL)
			return "int";


		type_or_kw_depth--;
		return "ptr";
}

inline void dump_params(FILE *f, param_t *start, char *spacer)
{
	param_t *p;
	int n;

	for(p = start; p != NULL; p = p->next) {
		n = 60 - strlen(p->name);
		spacer[n] = 0;
		fprintf(f, "    %s%s%s\n", p->name, spacer, type_or_kw(p->type));
		spacer[n] = ' ';
	}
}

inline void dump_enum(FILE *f, enum_element_t *start, char *spacer)
{
	enum_element_t *e;
	int n;

	for(e = start; e != NULL; e = e->next) {
		n = 60 - strlen(e->name);
		spacer[n] = 0;
		fprintf(f, "    %s%s%s\n", e->name, spacer, "0");
		spacer[n] = ' ';
	}
}


void dumplist(FILE *f)
{
	func_t *fun;
	event_t *e;
	keyword_t *k;
	enum_t *en;

	char spacer[128];
	char *s;
	int n;

	for(n=0,s=spacer; n<128; n++,s++)
		*s=' ';

	fprintf(f, "$$$Functions\n");
	for(fun = functions; fun != NULL; fun = fun->next) {
		n = 64 - strlen(fun->name);
		spacer[n] = 0;
		if (fun->type != direct)
			fprintf(f, "%s%s%s\n", fun->name, spacer, type_or_kw(fun->type));
		else
			fprintf(f, "%s%s-string\n", fun->name, spacer);
		spacer[n] = ' ';

		dump_params(f, fun->params, spacer);

		fprintf(f, "\n");
	}

	fprintf(f, "$$$Events\n");
	for(e = events; e != NULL; e = e->next) {
		n = 64 - strlen(e->name);
		spacer[n] = 0;
		fprintf(f, "%s%sint\n", e->name, spacer);
		spacer[n] = ' ';

		dump_params(f, e->params, spacer);

		fprintf(f, "\n");
	}

	fprintf(f, "$$$Keywords\n");
	for(k = keywords; k != NULL; k = k->next) {
		if (k->type == kw_type_export) {
			n = 63 - strlen(k->name);
			spacer[n] = 0;
			fprintf(f, "@%s%sint\n", k->name, spacer);
			spacer[n] = ' ';

			dump_params(f, k->params, spacer);

			fprintf(f, "\n");
		}
	}

	fprintf(f, "$$$Enums\n");
	for(en = enums; en != NULL; en = en->next) {
		fprintf(f, "%s\n", en->name);
		dump_enum(f, en->elements, spacer);
		fprintf(f, "\n");
	}

	printf("\n");
}

char *chr_insert(char *string, char c)
{
	memmove(string+1, string, strlen(string)+1);
	*string = c;
	return string;
}

int check_kw(char *name)
{
	return 0;
}

int main(int argc, char *argv[])
{
	int n;

	for(n=1; n<argc; n++) {
		if (strcmp(argv[n], "-v") == 0)
			verbose++;
	}
	yylgparse();
	verbprintf(1, "END.\n\n");
	dumplist(stdout);
	return 0;
}

void curr_params_push()
{
	param_stack_t *se;

	se = malloc(sizeof(param_stack_t));
	se->params = curr_params;
	se->next   = param_stack;

	param_stack = se;
}

void curr_params_pop()
{
	param_stack_t *se;

	assert((param_stack != NULL) && ("parameter stack underflow"));

	/* XXX: should free curr_params */

	se = param_stack;
	param_stack = se->next;

	curr_params	= se->params;
	free(se);
}

void enum_create(char *enum_name)
{
	enum_t *e;

	e = malloc(sizeof(enum_t));

	e->name     = strdup(enum_name);
	e->elements = NULL;
	e->next     = enums;


	enums = e;
}

struct enum_t *find_enum(char *enum_name)
{
	struct enum_t *e;
	for(e = enums; e != NULL; e = e->next)
	{
		if (strcmp(e->name, enum_name) == 0)
			return e;
	}
	return NULL;
}

void enum_add_element(char *element_name)
{
	enum_element_t *e;

	if ((element_name == NULL) || (enum_nowrap != 0))
		return;

	assert(enums != NULL);

	e = malloc(sizeof(enum_element_t));
	e->name = strdup(element_name);
	e->next = enums->elements;

	enums->elements = e;
}

void enum_close()
{
	/* Nothing to do yet */
}
